# coding=utf-8
from django import forms
from django.forms.util import ErrorList


class QueryForm(forms.Form):
    def __init__(self, data=None, files=None, auto_id='id_%s', prefix=None, initial=None, error_class=ErrorList,
                 label_suffix=':', empty_permitted=False, query=None):

        super(QueryForm, self).__init__(data, files, auto_id, prefix, initial, error_class, label_suffix,
            empty_permitted)
        self.query = query
        if self.query :
            for param in self.query.dicovery_params():
                self.fields[param] = forms.CharField()




# coding=utf-8
from django.core.urlresolvers import reverse
from django.db import models
from django.conf import settings
import re
from requery.local_settings import SQL_STATEMENTS_NOT_ALLOWED

PATTERN = re.compile(":(?P<var_name>[a-zA-Z0-9_]+)")
CHOICES_DATABASE = [(key, key) for key in settings.DATABASES.keys()]


class Query(models.Model):
    name = models.CharField(max_length=55)
    database = models.CharField(max_length=30, default='default', choices=CHOICES_DATABASE)
    text = models.TextField()

    def __unicode__(self):
        return self.name

    def dicovery_params(self):
        return PATTERN.findall(self.text)

    def get_absolute_url(self):
        return reverse('admin:requery_query_form', args=(self.id,))

    def prepare_text(self):
        params = self.dicovery_params()
        prepared = self.text
        for param in params:
            prepared = prepared.replace(':%s' % param, '%s')

        return prepared

    def is_allow(self):
        for statement in SQL_STATEMENTS_NOT_ALLOWED:
            if re.match('\s*' + statement + '\s', self.text, re.IGNORECASE):
                return False
        return True


# coding=utf-8
# Create your views here.
from django.conf import settings
from django.contrib.admin.models import LogEntry
from django.contrib.contenttypes.models import ContentType
from django.http import HttpResponse, HttpResponseForbidden
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template.defaultfilters import safe
from django.utils.encoding import force_unicode
from requery.forms import QueryForm
from requery.models import Query
from django.db import connections
from django.utils import simplejson


def form_query(request, query_id):
    query = Query.objects.get(id=query_id)
    form = QueryForm(initial={'text':query.text}, query=query)
    context = {
        'title': 'Run Query %s on database %s' % (force_unicode(query.name), query.database),
        'form': form,
        'object_id': query_id,
        'original': query,
        'media': safe("""<script type="text/javascript" src="{0}admin/js/core.js"></script>
                    <script type="text/javascript" src="{0}admin/js/jquery.js"></script>
                    <script type="text/javascript" src="{0}admin/js/jquery.init.js"></script>""".format(settings.STATIC_URL)),
        'is_popup': "_popup" in request.REQUEST,
        'app_label': query._meta.app_label,
        'opts': query._meta,
        'change': False,
        }
    return render_to_response('requery/query.html', context, context_instance=RequestContext(request))


def run_query(request, query_id):
    query = Query.objects.get(id=query_id)
    if query.is_allow():
        text = query.prepare_text()
        params = []
        for param in query.dicovery_params():
            params.append(request.POST[param])

        cursor = connections[query.database].cursor()
        cursor.execute(text, params)
        lines = []
        for line in cursor.fetchall():
            last_line = line
            lines.append([unicode(tup) for tup in line])

        data_types = []
        for item in last_line:
            data_types.append(item.__class__.__name__)

        if lines :
            columns = [col[0] for col in cursor.description]
            response = {
                'template' : '#table-response',
                'columns' : columns,
                'lines' : lines,
                'datatypes': data_types,
            }
        else:
            response = {
                'template' : '#message-response',
                'message' : 'No data'
            }
        cursor.close()
        LogEntry(user=request.user,
                 content_type=ContentType.objects.get_for_model(query),
                 object_id=query.id,
                 object_repr=force_unicode(query),
                 change_message="run with %s" % ', '.join(['%s:%s' % (key, value)
                                                 for key, value in request.POST.items()
                                                 if key != 'csrfmiddlewaretoken']),
                 action_flag=2 #change
        ).save()
        return HttpResponse(simplejson.dumps(response), mimetype='application/json')

    return HttpResponseForbidden()



# coding=utf-8
from django.conf.urls import url
from django.contrib import admin
from django.core.urlresolvers import reverse
from requery import views
from requery.models import Query
from requery import local_settings

try:
    import pygments
    from pygments import highlight
    from pygments.lexers import SqlLexer
    from pygments.formatters import HtmlFormatter
    from pygments.styles.vs import VisualStudioStyle
except ImportError as e:
    pygments = False


class QueryAdmin(admin.ModelAdmin):

    list_display = ('id', 'database', 'name', 'short_text', 'count_params', 'run')
    list_filter = ('database',)
    search_fields = ('name',)
    change_form_template = 'admin/requery/change_form_requery.html'

    if local_settings.SQL_STATEMENTS_NOT_ALLOWED:
        list_display += ('allow',)

    def short_text(self, query):
        if pygments:
            return highlight(query.text[:50], SqlLexer(), HtmlFormatter(style=VisualStudioStyle, noclasses=True))
        else:
            return query.text[:50]
    short_text.allow_tags = True

    def count_params(self, query):
        return len(query.dicovery_params())

    def allow(self, query):
        return query.is_allow()
    allow.boolean = True

    def get_urls(self):

        urls = super(QueryAdmin, self).get_urls()
        my_urls = [
            url(
                r'(?P<query_id>\d+)/running/',
                self.admin_site.admin_view(views.form_query),
                name='requery_query_form',
            ),url(
                r'(?P<query_id>\d+)/run/',
                self.admin_site.admin_view(views.run_query),
                name='requery_query_run',
            ),
        ]
        return my_urls + urls

    def run(self, query):
        if query.is_allow():
            return "<a href=%s>Run</a>" % reverse('admin:requery_query_form', args=(query.id,))
        return "<a href=%s>View</a>" % reverse('admin:requery_query_form', args=(query.id,))
    run.short_description = "Run"
    run.allow_tags = True


admin.site.register(Query, QueryAdmin)




# coding=utf-8
from django.conf import settings

# Define here statements that are not allowed in the querys you will run.
# Queries that contains this statements still can be stored by requery.
SQL_STATEMENTS_NOT_ALLOWED = getattr(settings, 'SQL_STATMENTS_NOT_ALLOWED', (

    )
)




# -*- coding:utf-8 -*-
from setuptools import setup, find_packages

setup(
    name='requery',
    version='0.4.0.0',
    url='http://github.com/ebertti/requery/',
    author='Ezequiel Bertti',
    author_email='ebertti@gmail.com',
    install_requires=['django>=1.7', 'Pygments>=1.6', 'six>=1.7',],
    packages=['requery'],
    include_package_data=True,
    license='MIT License',
    platforms=['OS Independent'],
    description="Simple way to store and use queries in database for use of DBA for Django Admin",
    long_description=(open('README.rst').read()),
    keywords='query dba repository report django',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Natural Language :: Portuguese (Brazilian)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',

        'Topic :: Database',
        'Topic :: Database :: Front-Ends',
    ],
    zip_safe=False,
)
